%-------------------Introduction to Elitist Genetic Algorithm (EGA) based Heavy Good Electric Vehicle (HGEV)------------------- 
%--------------------------------------------Charging Station Optimal Designer-------------------------------------------------
% Introducing the "HGEV Charging Station EGA-based Design Package": 
% This solution leverages an Elitist Genetic Algorithm (EGA) to tackle interconnected challenges in Energy Storage System (ESS) 
% design and demand-side management, particularly focusing on battery scheduling. It addresses these concerns simultaneously for 
% diverse styles of Heavy Goods Electrical Vehicle (HGEV) Depots and on-route charging stations. The ESS design encompasses 
% considerations like battery capacity and power electronic board rating power, while demand-side management involves a series of 
% battery charging/discharging power scheduling within the evaluation time window.
%------------------------------------------------------------------------------------------------------------------------------
%---   Application Guide:
%---1) Input Data Setup: Begin by setting the input data, including the station charging load profile (currently a daily power 
%      profile in half-hourly measurement resolution) and the dynamic tariff profile (a daily tariff variation in half-hourly 
%      resolution for the UK).
%---2) Energy Storage System (ESS) Characteristics: Define the fundamental features of the Energy Storage System (ESS), primarily 
%      focusing on Efficiency. Additionally, consider capacity and loading rate limits.
%---3) Station Connection Power Capacity: Introduce the maximum power capacity for the station connection, typically influenced by 
%      transformer rating power and cable rating current.
%---4) String Components and Measurement Resolution: Specify the number of input/output string components and the measurement time 
%      resolution.
%---5) ESS Cost Coefficients: Outline the ESS capital and operating cost coefficients, including Storage Capital Cost (SCC) in 
%      [£/kWh per day], Discharge Capital Cost (DCC) in [£/kW per day], Energy Fixed Operation and Maintenance (FOME) in 
%      [£/kWh per day], and Power Fixed Operation and Maintenance (FOMP) in [£/kW per day].
%---6) Elitist Genetic Algorithm (EGA) Control Parameters: Set the parameters for the Elitist Genetic Algorithm (EGA) control, 
%      including Cross Number (defining the optimization population - twice the cross number), Mutation (determining the number of 
%      population members subject to mutation), Max Generation (specifying the maximum iterations for the optimization method), and 
%      Error of Convergence (EC) (introducing the target error of results). The latter two factors act as constraints for halting the 
%      optimization process.
%---7) Execute the package and explore the generated results.
%------------------------------------------------------------------------------------------------------------------------------
clear;
clc;
close all
% ---------------------------------------------- Load Input Data---------------------------------------------------------------
load 12HoursDemandHalfHourly.mat     %---To Load Station Charging Demand-------------------------------------------------------
load DailyAverageMIBasedPrice.mat    %---To Load Tariff Profile----------------------------------------------------------------
InitialPopulation                    %---To load EGA Initial Population--------------------------------------------------------
%-------------------------------------------Introduce ESS Base Characteristics-------------------------------------------------
Efficiency=0.9;                      %---Introduce ESS Efficiency--------------------------------------------------------------
IniLoading=0.5;                      %---Initial Loading Introduced to chek the first time step feasibility--------------------
DeltaP= 1;                           %---Delta Positive in Power (Max Charger/Electrolyser incremental loading capability)-----
DeltaN=-1;                           %---Delta Negative in Power (Max Charger/Electrolyser decremental loading capability)-----
%-------------------------------------------Introduce Connection Limits--------------------------------------------------------
CRP=6000;                            %---Connection Rating Power (CRP)/ Transformer Rating Power [kW]--------------------------
%----------------------------------------------------System Time Resolusion----------------------------------------------------
totalparameters = 48 ;               %---The number of time string elements (Charging Demand or Tariff Profile)----------------
TC=0.5;                              %---Measurement Time Constant [h]---------------------------------------------------------
%----------------------------------Introduce ESS Capital and Operating Cost Coefficient----------------------------------------
SCC=0.0286;                          %---Storage Capital Cost (SCC) [£/kWh-day]------------------------------------------------
DCC=0.0251;                          %---Discharge Capital Cost (DCC) [£/kW-day]-----------------------------------------------
FOME=0.0005;                         %---Energy Fixed Operation and Maintenece (FOME) [£/kWh-day]------------------------------
FOMP=0.00018;                        %---Power Fixed Operation and Maintenece (FOMP) [£/kW-day]--------------------------------
%----------------------------------------------------EGA Control Parameters----------------------------------------------------
crossno = 200;                       %---Cross Number Defines the Optimization Population--------------------------------------                        
mutnum = 40;                         %---Mutation Defines the number of members subject to Mutation----------------------------
MAXGENERATIONNO = 15000;             %---Defines Maximum Generation as an Stop Condition---------------------------------------
EC=0.00;                             %---Defines the target Error at Convergence point as another stop condition---------------
%---------------------------------------------The Next Steps in the EGA Design Process-----------------------------------------
InputsPreProcessing;                 %---Step 1) Pre-Processing the Inputs-----------------------------------------------------
InputsDepiction;                     %---Step 2) Plot the Inputs---------------------------------------------------------------
EGALoop;                            %---Step 3) Execute the Optimization Approach for the Case--------------------------------
PostProcessingResults;               %---Step 4) Post-Processing the Optimization Outputs--------------------------------------
OutputsDepiction                     %---Step 5) Visualise the Process Outputs-------------------------------------------------
DecisionsFinalEvaluation             %---Step 6) Validate the Final Results by Re-Evaluation-----------------------------------
%------------------------------------------------------------------------------------------------------------------------------
disp('The Process Completed')        %---Step 7) Display Process Completion Message--------------------------------------------